package customersvc

import (
	"errors"

	"github.com/google/uuid"

	mgo "gopkg.in/mgo.v2-unstable"
	"gopkg.in/mgo.v2-unstable/bson"
)

const (
	CustomerColl = "customer"
)

var (
	ErrNotFound = errors.New("Not Found!")
)

type Servicer interface {
	Store(c *Customer) error
	FetchByID(id string) (*Customer, error)
}

// Service customer service
type Service struct {
	DS *mgo.Database
}

// Store customer
func (s *Service) Store(c *Customer) error {

	c.ID = uuid.New().String() // could collide
	return s.coll().Insert(c)
}

// FetchById gets by object id
func (s *Service) FetchByID(id string) (*Customer, error) {

	c := Customer{}
	err := s.coll().Find(bson.M{"_id": id}).One(&c)
	if err == mgo.ErrNotFound {
		return nil, ErrNotFound // This is because the implementers of this service should be database agnostic
	}
	if err != nil {
		return nil, err
	}
	return &c, nil
}

func (s *Service) coll() *mgo.Collection {
	return s.DS.C("customer")
}
